#ifndef LYN_H
#define LYN_H

#include "struct.h"

#define version_string "LYN v0.8a.9L"
#define bare_version "0.8a.9L"                         /* for cddb */
#define copyright_string "(c) Copyright 1995-97, Devin Carraway"

/* #define DEBUGLEVEL 0 */

#ifdef SQL_DBASE
#define SQL_DBASE_SUBMIT
#define SQL_DBASE_SERVER "sql.sonoma.net"
#define DEFAULT_SQL_SERVER "sql.sonoma.net"
#endif

#define CDDB_DBASE

#define DEFAULT_CDDB_SERVERS "cddb.moonsoft.com cddb.sonic.net sunsite.unc.edu"

#define DEFAULT_CDPATH "/usr/local/lib/lyn"
#define CYCLE_SLEEP_USECS (400000L)

#define XTERM_TLINE

#define CD_MIXER_DEV (SOUND_MIXER_CD)
#define DEFAULT_MIXER "/dev/mixer"
#define DEFAULT_CD_DEVICE "/dev/cdrom"

#define DIRECT_V
#define EJECT_TRAY 0
#define RESET 2
#define CLOSE_TRAY 5
#define DATA_TRACK 64
#define LOCK 1
#define UNLOCK 0

#define PLAY_DISK 0
#define PLAY_TRACK 1
#define PLAY_SHUFFLE 2

#define IF_LISTMODE_NORM (listmode==_LISTMODE_NORM)
#define IF_LISTMODE_EXT (listmode==_LISTMODE_EXT)
#define LISTMODE_LEFTX ((IF_LISTMODE_EXT)?(27):(61))

#define _LISTMODE_NORM 0
#define _LISTMODE_EXT 1

#define _PLAYMODE_NORM 0
#define _PLAYMODE_TRACKLOOP 1
#define _PLAYMODE_SHUFFLE 2
#define _PLAYMODE_DISCLOOP 3


#define TRACKLIST_SIZE (abs(tracklist_bottom-tracklist_top))
#define _LOC_TRACKLIST_END (((7+TRACKLIST_SIZE)>23)?23:7+TRACKLIST_SIZE)

#define unhighlight_tracknums() highlight_track(0,0)
#define gotoxy(x,y) goxy(x-1,y-1)
#define goxy(x,y) move(y,x)

#define DRAWVOL() drawvol()

#define perror(s) lperror(s,3)

#define _CFG_VERBOSITY_SILENT 0
#define _CFG_VERBOSITY_NORMAL 1
#define _CFG_VERBOSITY_HIGH 2
#define _CFG_TRACKSUMSTART_NORM 1
#define _CFG_TRACKSUMSTART_SL 2
#define _CFG_TRACKSUMSTART_DEBUG 3
#define _CFG_FORCEVOLUME_NONE -1
#define _CFG_WHICHVOLUME_CD 1
#define _CFG_WHICHVOLUME_MIXER 2
#define _CFG_WHICHVOLUME_BOTH (_CFG_WHICHVOLUME_CD | _CFG_WHICHVOLUME_MIXER)
#define _CFG_WHICHVOLUME_DEFAULT _CFG_WHICHVOLUME_CD

#define NUM_CONFIGS 50

typedef struct {
	char	*name;
	int	type;
	char	*cval;
	int	ival;
	float	fval;
} config_setting;

typedef struct {
  char		verbosity;
  char	 	tracksumstart;
  char		active_cdromdevice[160];
  char		*cdromdevices;
  signed int 	forcevolume;
  char		whichvolume;
  char		activevolume;
  char		dborder[80];
  char		*cddbservers;
  char		sqlcopytolocal;
  char		cddbcopytolocal;
  char		configfiles;
  char 		cddbpath[160];
  config_setting settings[NUM_CONFIGS];
} configrec;

struct cline {
	char key[40];
	char val[80];
};

/* hardware.c */

unsigned long cd_head_position(void);
void cd_get_volume(volumeinfo *vol);
void cd_set_volume(volumeinfo *vol);
short cd_getupc(void);
void cd_get_audio_info(void);
void cd_set_track(short tracknum);
void cd_track_length(short tracknum, unsigned char *min, unsigned char *sec, unsigned char *frame);
int cd_status(void);
void cd_seek(unsigned long location);
void cd_play_audio(unsigned long begin, unsigned long end);
void cd_stop_audio(void);
void cd_resume_audio(void);
void cd_cmd(unsigned char mode);
void cd_getpos(playinfo *info);
short cdrom_installed(void);
short init_drive(unsigned char drivenum);
short cd_done_play(void);
short cd_mediach(void);
int device_available(const char *fn);
int mix_open();
void mix_close();
int mix_set_volume(int req, int *left, int *right);
int mix_get_volume(int req, int *left, int *right);
/* keep these: */
int cdv_getvol(int *left, int *right);
int cdv_setvol(int *left, int *right);
int mix_getvol(int *left, int *right);
int mix_setvol(int *left, int *right);
int read_volumes(void);
int set_vol(int leftval, int rightval);
int update_volumes(void);

void where_the_fuck_are_we(void);
unsigned char *charstr(int len, unsigned char rc);
void vmode(void);
int strcmpi(const char *s, const char *s1);
void cstr(unsigned x, unsigned y, char attrib, unsigned len);
void cwrite(unsigned x, unsigned y, char attrib, char *fmt, ...);
void dwrite(unsigned x, unsigned y, char attrib, char *fmt, ...);
void cnwrite(int y, char attrib, char *fmt, ...);
void init_curses(void);
void f_write(unsigned x, unsigned y, char attrib, char *str);
void show_status(void);
void show_mode(void);

void newdisk(void);
void nodisk(void);
short play_song(int num);
void highlight_track(int mode, unsigned int track);
void hopseek(signed int seconds);
void pause_song(void);





void edit_songdata(void);
void ccc_draw(int cur);
int write_dbase(void);
int read_dbase(void);
void cd_lock(int d);
struct cdinfo * read_toc(void);
void play_chunk( int start, int end );
void drawvol(void);
void drawvol_force(void);
void drawvol_real(int force);
void test_toc(void);
void update_topline(void);

/* config.c */
void log_config_debug(char *fmt, ...);
int config_line(char *ss, struct cline *cc);
int overlay_config(char *fn, configrec *cfg);
void parse_cdrom_devices(configrec *cfg);
int read_config(configrec *cfg);
void blank_config(void);
void prep_new_config(void);
void clear_config(int new);
int store_config(char *name, char *value);
char *aconfig(char *name);
signed int iconfig(char *name);

/* util.c */
void sig_winch_handler(int snum);

/* dbase.c */
int write_local_dbase(int id, struct cdinfo *cdd);
int read_local_dbase(int it, struct cdinfo *cdd);
int blank_tracknames(void);
int blank_trackdata(void);

/* sqldbase.c */
char *msqlEscape(const char *str);
int sql_submit_cd(const int id, const struct cdinfo *cdd);
int sql_lookup_cd(int id, struct cdinfo *cdd);
void log_sql_debug(char *fmt, ...);

/* cddbdbase.c */
unsigned long cddb_discid(int tot_trks);
int cddb_sum(int n);
char *gen_cddb_id(void);
void log_cddb_debug(char *fmt, ...);
int read_cddb_dbase(char *id, struct cdinfo *cdd);
int read_cddb_dbase_s(char *id, struct cdinfo *cdd, char *server);

/* cosmetic.c */
void setup_screen(int ccc);
void draw_menu(int num);
void report_status(char *str);
void lperror(char *prefix, int sec);
void lperror(char *prefix, int sec);
void report_err(char *str, int sec);
int clear_err(void);
int expire_err(void);
void report_topline(int c, char *str);
void update_topline(void);
int update_xtermtitle(int playing, int min, int sec);
void redraw_screen(void);
void draw_tracklist(void);
void blank_tracklist(void);
char *last_token(char delim, char *str);

configrec config;
cd_datarec dd;
int curses_ready;
int cur_index, cur_frame, cur_track, menunum,ccc;
short screeny, ctr, done, diskin, playing, paused, _fast, listmode;
char oldstatus,ts,tc,sourcepath[81];
cdrom_datarec cdrom_data;
unsigned char bal;
int vol;
int /*cur_track, */cur_pos_abs, cur_pos_rel, cur_tracklen, cur_cdlen, cur_cdmode,
  cur_ntracks, cur_lasttrack, cur_firsttrack, cur_listno;
struct cdinfo thiscd,*cd;
unsigned char playmode;
unsigned tracklist_top,tracklist_bottom, lasttrack, ch;
struct volumes volume;
#else

extern configrec config;
extern cd_datarec dd;
extern int curses_ready;
extern int cur_index,cur_frame,cur_track,menunum,ccc;
extern short screeny, ctr, done, diskin, playing, paused, _fast, listmode;
extern char oldstatus,ts,tc,sourcepath[81];
extern cdrom_datarec cdrom_data;
extern unsigned char bal;
extern int vol;
extern int /*cur_track, */cur_pos_abs, cur_pos_rel, cur_tracklen, cur_cdlen, cur_cdmode,
  cur_ntracks, cur_lasttrack, cur_firsttrack, cur_listno;
extern struct cdinfo thiscd,*cd;
extern unsigned char playmode;
extern unsigned tracklist_top,tracklist_bottom, lasttrack, ch;
extern struct volumes volume;
#endif


